const fs = require('fs');
const path = require('path');
const utils = require("../utils")
const findModal = require('./findModal');
const tempList = require('./constant');

const yarn = findModal('yarn');
const git = findModal('git');

let fileCount = 0; // 文件数量
let dirCount = 0; // 文件夹数量
let flat = 0; // readir数量
let isInstall = false;

module.exports = async function (answer) {
    utils.greenBright('------开始构建------');
    const cwdPath = process.cwd();
    const tempOpt = tempList.find(item => item.name === answer.tempName);
    const tempPath = __dirname.slice(0, -3) + 'template';
    const sourcePath = `${tempPath}/${tempOpt.name}`;
    await getTemplate(tempPath, sourcePath, tempOpt);
    const currentPath = folderExist(cwdPath, answer);
    const dir = fs.opendirSync(currentPath);
    for await (const dirent of dir) {
        if (dirent) {
            utils.redBright(`Error：目录'${currentPath}'下已存在文件或目录，请手动处理后再进行操作`)
            return;
        }
    }
    revisePackageJson(answer, sourcePath, currentPath).then(() => {
        utils.green('------开始初始化项目------');
        copyMethod(sourcePath, currentPath, () => {
            utils.green('------初始化项目成功------');
            if (!isInstall) {
                isInstall = true;
                utils.greenBright('------开始install------');
                yarn([''], { cwd: currentPath }, () => {
                    utils.greenBright('------完成install------');
                    // runProject();
                })
            }
        });
    })
}

function folderExist(cwdPath, answer) {
    const { name } = answer;
    const upperFolder = cwdPath.split(path.sep).pop();
    if (name === upperFolder) return cwdPath;
    const currentPath = `${cwdPath}/${name}`;
    !fs.existsSync(currentPath) && fs.mkdirSync(currentPath);
    return currentPath;
}

// git clone
function getTemplate(tempPath, sourcePath, tempOpt = {}) {
    return new Promise((resolve) => {
        const { name, gitPath } = tempOpt;
        if (fs.existsSync(sourcePath)) {
            utils.green('------模板文件更新开始------');
            git(['pull'], { cwd: sourcePath }, () => {
                utils.green('------模板文件更新完成------');
                resolve();
            })
        } else {
            utils.green('------模板文件下载开始------');
            git([`clone`, gitPath], { cwd: tempPath }, () => {
                utils.green('------模板文件下载完成------');
                resolve();
            });
        }
    });
}

function revisePackageJson(answer, sourcePath, currentPath) {
    return new Promise((resolve) => {
        fs.readFile(sourcePath + '/package.json', (err, data) => {
            if (err) throw err;
            const { author, name, description } = answer;
            let json = JSON.parse(data.toString());
            json.name = name.trim() || 'name';
            json.author = author.trim() || 'author';
            json.description = description.trim() || 'description';
            const packagePath = currentPath + '/package.json';
            fs.writeFile(packagePath, Buffer.from(JSON.stringify(json, null, 2)), () => {
                utils.green('创建文件' + path);
                resolve();
            })
        })
    })
}

/**
 * 
 * @param {*} sourcePath //template资源路径 
 * @param {*} currentPath //当前项目路径
 * @param {*} copyCb //项目复制完成回调函数
 */
function copyMethod(sourcePath, currentPath, copyCb) {
    flat++;
    fs.readdir(sourcePath, (err, paths) => {
        flat--;
        if (err) throw err;
        paths.forEach(path => {
            if (path !== '.git' && path !== 'package.json') fileCount++;
            const newSourcePath = sourcePath + '/' + path;
            const newCurrentPath = currentPath + '/' + path;
            fs.stat(newSourcePath, (err, stat) => {
                if (err) throw err;
                if (stat.isFile() && path !== 'package.json') {
                    const readStream = fs.createReadStream(newSourcePath);
                    const writeSteam = fs.createWriteStream(newCurrentPath);
                    readStream.pipe(writeSteam);
                    utils.yellowBright('创建文件：' + newCurrentPath);
                    fileCount --;
                    completeControl(copyCb);
                } else if (stat.isDirectory()) {
                    if (path !== '.git' && path !== 'package.json') {
                        dirCount++;
                        dirExist(newSourcePath, newCurrentPath, copyCb);
                    }
                }
            })
        })
    })
}

/**
 * 
 * @param {*} sourcePath //template资源路径
 * @param {*} currentPath // 当前项目路径
 * @param {*} copyCb // 复制完成回调函数
 */
function dirExist(sourcePath, currentPath, copyCb) {
    if (fs.existsSync(currentPath)) {
        copyMethod(sourcePath, currentPath, copyCb);
    } else {
        fs.mkdir(currentPath, () => {
            fileCount--;
            dirCount--;
            copyMethod(sourcePath, currentPath, copyCb);
            completeControl(copyCb);
        })
    }
}

// 处理copy回调
function completeControl(copyCb) {
    if (fileCount === 0 && dirCount === 0 && flat ===0) {
        utils.greenBright('------构建完成------');
        copyCb && copyCb();
    }
}