import React from 'react';
import {
  Layout,
  Image,
  Menu,
} from 'antd';
import { Link } from 'react-router-dom';
import { RouteType } from 'src/routes';

export interface SidebarPropsType {
  collapsed: boolean;
  routes: RouteType[];
}

const { Sider } = Layout;
const { SubMenu } = Menu;

export default function Sidebar({
  collapsed,
  routes,
}: SidebarPropsType): JSX.Element {
  const createLinks = () => routes.map((prop, key) => {
    if (prop.subMenu) {
      return (
        <SubMenu key={key.toString()} icon={<prop.icon />} title={prop.name}>
          {prop.subMenu.map((route, routeKey) => (
            <Menu.Item key={key.toString() + routeKey.toString()} icon={<route.icon />}>
              <Link to={route.layout + route.path}>
                {route.name}
              </Link>
            </Menu.Item>
          ))}
        </SubMenu>
      );
    }
    if (prop.isSidemenu) {
      return (
        <Menu.Item key={key.toString()} icon={<prop.icon />}>
          <Link to={prop.layout + prop.path}>
            {prop.name}
          </Link>
        </Menu.Item>
      );
    }
    return null;
  });
  return (
    <Sider
      style={{
        overflow: 'auto',
        height: '100vh',
        position: 'fixed',
        left: 0,
      }}
      breakpoint="sm"
      collapsedWidth="0"
      collapsible
      collapsed={collapsed}
      trigger={null}
      theme="light"
    >
      <Image
        className={collapsed ? 'ml2 my2' : 'ml3 pl2 my2'}
        width={collapsed ? 50 : 100}
        src=""
      />
      <Menu
        defaultSelectedKeys={['1']}
        defaultOpenKeys={['sub1']}
        mode="inline"
        theme="light"
      >
        {createLinks()}
      </Menu>
    </Sider>
  );
}
