import React, { Component } from 'react';
import {
  Layout,
} from 'antd';
import {
  Route,
  Switch,
  RouteComponentProps,
  Redirect,
} from 'react-router-dom';
import Routes, { RouteType } from 'src/routes';
import Header from './components/Header';
import Footer from './components/Footer';
import Sidebar from './components/Sidebar';

const { Content } = Layout;

export default class AdminLayout extends Component<RouteComponentProps, { collapsed: boolean }> {
  constructor(props: RouteComponentProps) {
    super(props);
    this.state = {
      collapsed: false,
    };
  }

    getRoutes = (layoutRoutes: RouteType[]): null | any => layoutRoutes.map((prop, key) => {
      // if a route doesn't have any subMenu and its layout is admin
      const propPath = prop.layout + prop.path;
      if (prop.layout === '/admin' && !prop.subMenu) {
        return (
          <Route
            exact
            path={propPath}
            component={(): JSX.Element => <prop.component />}
            key={key.toString()}
          />
        );
        // For route that has suMenu, loop each item and find the exact route
      } if (prop.subMenu) {
        return (
          prop.subMenu !== undefined
                    && prop.subMenu.map((sub, idx) => (
                      <Route
                        exact
                        path={sub.layout + sub.path}
                        component={sub.component}
                        key={idx.toString()}
                      />
                    ))
        );
      }
      return <Redirect to="/admin/error/404" key={key.toString()} />;
    });

    toggleCollapsed = (): void => {
      this.setState(({ collapsed }) => ({ collapsed: !collapsed }));
    };

    getCurrentRouteText = (path: string, thisRoutes: RouteType[]): string => {
      for (let i = 0; i < thisRoutes.length; i += 1) {
        const menu = thisRoutes[i];
        if (path.includes(menu.layout + menu.path) && !menu.subMenu) {
          return menu.name;
        }
        if (menu.subMenu) {
          for (let j = 0; j < menu.subMenu.length; j += 1) {
            const subMenu = menu.subMenu[j];
            if (path === subMenu.layout + subMenu.path) {
              return subMenu.name;
            }
          }
        }
      }
      return 'Brand';
    };

    render(): JSX.Element {
      const { collapsed } = this.state;
      const { location } = this.props;

      return (
        <>
          <Layout>
            <Sidebar collapsed={collapsed} routes={Routes} />
            <Layout className="site-layout" style={{ marginLeft: collapsed ? 0 : 200 }}>
              <Header
                currentRouteText={this.getCurrentRouteText(location.pathname, Routes)}
                collapsed={collapsed}
                routes={Routes}
                toggleCollapsed={this.toggleCollapsed}
              />
              <Content style={{ margin: '24px 16px 0', overflow: 'initial' }}>
                <div style={{ padding: 24, minHeight: 360 }}>
                  <Switch>{this.getRoutes(Routes)}</Switch>
                </div>
              </Content>
              <Footer />
            </Layout>
          </Layout>
        </>
      );
    }
}
