import axios, {
  AxiosResponse,
  AxiosRequestConfig,
} from 'axios';

axios.interceptors.request.use((config) => config, (error) => Promise.reject(error));

axios.interceptors.response.use(({
  data: {
    status,
    data,
    msg,
  },
}) => (!status ? data : Promise.reject(msg)), (error) => Promise.reject(error));

function fetcher(
  methodAndUrl: string,
  params?: Record<string, any>,
  config?: AxiosRequestConfig,
): Promise<AxiosResponse<any>> {
  const [
    method,
    url,
  ] = methodAndUrl.split(' ');

  switch (method) {
    case 'POST': {
      return axios.post(url, params, config);
    }
    case 'PUT': {
      return axios.put(url, params, config);
    }
    case 'PATCH': {
      return axios.patch(url, params, config);
    }
    case 'DELETE': {
      return axios.delete(url, {
        params,
        ...config,
      });
    }
    case 'GET':
    default: {
      return axios.get(url, {
        params,
        ...config,
      });
    }
  }
}
export default fetcher;
